import React, { useState, useRef, useEffect, createRef, useCallback } from 'react';
import { Select } from '@highlight-ui/select';
import { parseMetadata, suffixMetadata, noop } from '@highlight-ui/utils-commons';
import { useClickOutside, useForkRef } from '@highlight-ui/utils-hooks';
import classnames from 'classnames';
import { Tag } from '@highlight-ui/tag';
import { InputContainer } from '@highlight-ui/input';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
var styles = {
  "heading-xsmall": "TagInput-module__DnmTWydo__v4-2-106",
  "heading-small": "TagInput-module__hZMUPoJE__v4-2-106",
  "heading-medium": "TagInput-module__HvjFnotz__v4-2-106",
  "heading-large": "TagInput-module__jUKtgoJG__v4-2-106",
  "heading-xlarge": "TagInput-module__RQVZf6Oz__v4-2-106",
  "heading-2xl": "TagInput-module__GA-o0jKc__v4-2-106",
  "body-small": "TagInput-module__k33MeLE0__v4-2-106",
  "body-base": "TagInput-module__wZQhTNV0__v4-2-106",
  "inputElement": "TagInput-module__oyYfVM94__v4-2-106",
  "body-large": "TagInput-module__JK-aoLTu__v4-2-106",
  "body-strong-small": "TagInput-module__tPSHR0-d__v4-2-106",
  "body-strong-base": "TagInput-module__n30CUmlJ__v4-2-106",
  "body-strong-large": "TagInput-module__fIMpWwrh__v4-2-106",
  "page-heading": "TagInput-module__s1ATSOdy__v4-2-106",
  "button": "TagInput-module__J9JaGA3I__v4-2-106",
  "form-label": "TagInput-module__w3dnyEyO__v4-2-106",
  "caption": "TagInput-module__-qC2Askb__v4-2-106",
  "text-link-default": "TagInput-module__x6EI6jl-__v4-2-106",
  "text-link-hover": "TagInput-module__zKpqozSQ__v4-2-106",
  "table-data-regular": "TagInput-module__O7mbrxNh__v4-2-106",
  "table-data-bold": "TagInput-module__tbikoIFV__v4-2-106",
  "container": "TagInput-module__HBdO6c6u__v4-2-106",
  "inputOutline": "TagInput-module__wf4JEsET__v4-2-106",
  "tagsContainer": "TagInput-module__0qdJURal__v4-2-106",
  "focused": "TagInput-module__hv8bWfUt__v4-2-106",
  "isHidden": "TagInput-module__x6-QjeAp__v4-2-106",
  "additionalTags": "TagInput-module__O6z716UX__v4-2-106"
};
var getTagsShownNumber = function (tagElements, containerElement) {
  if (!tagElements.length || !containerElement.current) return 0;
  var maxWidth = containerElement.current.offsetWidth;
  var additionalWidth = 4; /* tagElement left/right margin */
  var value = tagElements.slice(1).reduce(function (_a, _b) {
    var items = _a.items,
      totalWidth = _a.totalWidth;
    var current = _b.current;
    if (!current) return {
      items: items,
      totalWidth: totalWidth
    };
    // approximate width for the tag number of hidden tags
    var tagNumberLength = tagElements.length - items > 9 ? 40 : 30;
    // last item doesn't need TagNumber length
    var calculatedMaxWidth = items === tagElements.length - 1 ? maxWidth : maxWidth - tagNumberLength;
    // Width for one tag and any additional width
    var calculatedTagWidth = totalWidth + current.offsetWidth + additionalWidth;
    if (calculatedMaxWidth > calculatedTagWidth) {
      return {
        items: items + 1,
        totalWidth: calculatedTagWidth
      };
    }
    return {
      items: items,
      totalWidth: totalWidth + current.offsetWidth
    };
  }, {
    items: 1,
    totalWidth: tagElements[0].current ? tagElements[0].current.offsetWidth + additionalWidth : 0
  });
  return value.items;
};
var TagInput = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b;
  var tags = _a.tags,
    _c = _a.onInputClick,
    onInputClick = _c === void 0 ? noop : _c,
    onInputChange = _a.onInputChange,
    onTagAdd = _a.onTagAdd,
    onTagRemove = _a.onTagRemove,
    suffix = _a.suffix,
    className = _a.className,
    metadata = _a.metadata,
    selectRef = _a.selectRef,
    focused = _a.focused,
    outline = _a.outline;
  var _d = useState(''),
    inputValue = _d[0],
    setInputValue = _d[1];
  var _e = useState(false),
    isFocused = _e[0],
    setFocus = _e[1];
  var _f = useState([]),
    elRefs = _f[0],
    setElRefs = _f[1];
  var inputRef = useRef(null);
  var tagContainerRef = useRef(null);
  var clearInput = function () {
    setInputValue('');
    if (onInputChange) {
      onInputChange('');
    }
  };
  // TODO: refactor and remove selectRef from TagInput [PPUI-2030]
  var clickOutsideRef = useClickOutside(function (event) {
    var _a;
    setFocus(false);
    if (inputValue) {
      if (!selectRef || selectRef && !((_a = selectRef.current) === null || _a === void 0 ? void 0 : _a.contains(event.target))) {
        onTagAdd(inputValue);
      }
      clearInput();
    }
  });
  var containerRef = useForkRef([ref, clickOutsideRef]);
  var _g = useState(0),
    tagsShownNumber = _g[0],
    setTagsShownNumber = _g[1];
  useEffect(function () {
    setElRefs(function (prevState) {
      return Array(tags.length).fill(0).map(function (_, i) {
        return prevState[i] || /*#__PURE__*/createRef();
      });
    });
  }, [tags]);
  useEffect(function () {
    setTagsShownNumber(getTagsShownNumber(elRefs, tagContainerRef));
  }, [elRefs, tagContainerRef]);
  return /*#__PURE__*/React.createElement("div", __assign({
    className: classnames(styles.container, className)
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement(InputContainer, {
    focused: focused,
    outline: outline,
    suffix: suffix,
    containerRef: containerRef,
    className: styles.inputOutline,
    inputClassName: classnames(styles.tagsContainer, (_b = {}, _b[styles.focused] = isFocused, _b))
  }, /*#__PURE__*/React.createElement("div", {
    ref: tagContainerRef
  }, tags.map(function (tag, index) {
    var _a;
    return (
      /*#__PURE__*/
      // eslint-disable-next-line react/no-array-index-key
      React.createElement(React.Fragment, {
        key: index
      }, /*#__PURE__*/React.createElement(Tag, __assign({}, tag, {
        metadata: suffixMetadata(metadata, "tag-" + index),
        ref: elRefs[index],
        className: classnames(styles.tag, (_a = {}, _a[styles.isHidden] = !isFocused && index >= tagsShownNumber, _a)),
        onRemove: function (e) {
          e.stopPropagation();
          onTagRemove(tag, index);
        }
      })), !isFocused && index === tagsShownNumber - 1 && tags.length > tagsShownNumber && /*#__PURE__*/React.createElement(Tag, {
        className: styles.additionalTags,
        text: "+" + (tags.length - tagsShownNumber)
      }))
    );
  }), /*#__PURE__*/React.createElement("input", {
    ref: inputRef,
    className: styles.inputElement,
    value: inputValue,
    onKeyDown: function (event) {
      var value = event.currentTarget.value;
      if (value) {
        if (event.key === 'Enter' || event.key === 'Tab' || event.key === ',') {
          onTagAdd(value);
          clearInput();
        }
      }
    },
    onChange: function (event) {
      var input = event.target.value;
      if (input !== ',') {
        setInputValue(input);
        if (onInputChange) {
          onInputChange(input);
        }
      }
    },
    onFocus: function () {
      if (!isFocused) {
        setFocus(true);
        onInputClick();
      }
    }
  }))));
});
function TagPickerWithRef(_a, ref) {
  var tags = _a.tags,
    onInputChange = _a.onInputChange,
    onTagAdd = _a.onTagAdd,
    onTagRemove = _a.onTagRemove,
    options = _a.options,
    className = _a.className,
    metadata = _a.metadata,
    suffix = _a.suffix;
  var _b = useState(''),
    highlightedText = _b[0],
    setHighlightedText = _b[1];
  var handleOnInputChange = useCallback(function (input) {
    setHighlightedText(input);
    if (onInputChange) {
      onInputChange(input);
    }
  }, [onInputChange]);
  var renderTriggerCallback = useCallback(function (_a) {
    var onClick = _a.onClick,
      triggerRef = _a.ref,
      selectRef = _a.selectRef;
    return /*#__PURE__*/React.createElement(TagInput, {
      metadata: suffixMetadata(metadata, 'tag-input'),
      tags: tags,
      onInputClick: onClick,
      onInputChange: handleOnInputChange,
      onTagAdd: onTagAdd,
      onTagRemove: onTagRemove,
      suffix: suffix,
      ref: triggerRef,
      selectRef: selectRef
    });
  }, [handleOnInputChange, metadata, onTagAdd, onTagRemove, suffix, tags]);
  return /*#__PURE__*/React.createElement(Select, {
    ref: ref,
    className: className,
    metadata: metadata,
    renderTrigger: renderTriggerCallback,
    closeOnSelect: true,
    highlightedText: highlightedText,
    searchable: false,
    options: options,
    renderEmptyState: function () {
      return null;
    },
    onSelect: function (option) {
      return onTagAdd(option[0].label);
    }
  });
}
TagPickerWithRef.displayName = 'TagPicker';
var TagPicker = /*#__PURE__*/React.forwardRef(TagPickerWithRef);
export { TagInput, TagPicker };
//# sourceMappingURL=index.js.map
